//
// Copyright 2022 Vulcanize, Inc.
//
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import { createLibp2p } from 'libp2p';
import wrtc from 'wrtc';
import { hideBin } from 'yargs/helpers';
import yargs from 'yargs';
import fs from 'fs';
import path from 'path';
import debug from 'debug';
import { noise } from '@chainsafe/libp2p-noise';
import { mplex } from '@libp2p/mplex';
import { webRTCStar } from '@libp2p/webrtc-star';
import { floodsub } from '@libp2p/floodsub';
import { pubsubPeerDiscovery } from '@libp2p/pubsub-peer-discovery';
import { createFromJSON } from '@libp2p/peer-id-factory';
import { DEFAULT_SIGNAL_SERVER_URL } from './index.js';
import { HOP_TIMEOUT, PUBSUB_DISCOVERY_INTERVAL, PUBSUB_SIGNATURE_POLICY } from './constants.js';
import { multiaddr } from '@multiformats/multiaddr';
var log = debug('laconic:relay');
function main() {
    return __awaiter(this, void 0, void 0, function () {
        var argv, peerId, peerIdFilePath, peerIdObj, peerIdJson, wrtcStar, node, relayPeersFilePath, relayPeersListObj, relayPeersList;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    argv = _getArgv();
                    if (!argv.signalServer) {
                        console.log('Using the default signalling server URL');
                    }
                    if (!argv.peerIdFile) return [3 /*break*/, 2];
                    peerIdFilePath = path.resolve(argv.peerIdFile);
                    console.log("Reading peer id from file ".concat(peerIdFilePath));
                    peerIdObj = fs.readFileSync(peerIdFilePath, 'utf-8');
                    peerIdJson = JSON.parse(peerIdObj);
                    return [4 /*yield*/, createFromJSON(peerIdJson)];
                case 1:
                    peerId = _a.sent();
                    return [3 /*break*/, 3];
                case 2:
                    console.log('Creating a new peer id');
                    _a.label = 3;
                case 3:
                    wrtcStar = webRTCStar({ wrtc: wrtc });
                    return [4 /*yield*/, createLibp2p({
                            peerId: peerId,
                            addresses: {
                                listen: [
                                    argv.signalServer || DEFAULT_SIGNAL_SERVER_URL
                                ]
                            },
                            transports: [
                                wrtcStar.transport
                            ],
                            connectionEncryption: [noise()],
                            streamMuxers: [mplex()],
                            pubsub: floodsub({ globalSignaturePolicy: PUBSUB_SIGNATURE_POLICY }),
                            peerDiscovery: [
                                pubsubPeerDiscovery({
                                    interval: PUBSUB_DISCOVERY_INTERVAL
                                })
                            ],
                            relay: {
                                enabled: true,
                                hop: {
                                    enabled: true,
                                    timeout: HOP_TIMEOUT
                                },
                                advertise: {
                                    enabled: true
                                }
                            },
                            connectionManager: {
                                autoDial: false
                            }
                        })];
                case 4:
                    node = _a.sent();
                    console.log("Relay node started with id ".concat(node.peerId.toString()));
                    console.log('Listening on:');
                    node.getMultiaddrs().forEach(function (ma) { return console.log(ma.toString()); });
                    console.log();
                    // Listen for peers connection
                    node.addEventListener('peer:connect', function (evt) {
                        // console.log('event peer:connect', evt);
                        // Log connected peer
                        var connection = evt.detail;
                        log("Connected to ".concat(connection.remotePeer.toString(), " using multiaddr ").concat(connection.remoteAddr.toString()));
                    });
                    // Listen for peers disconnecting
                    node.addEventListener('peer:disconnect', function (evt) {
                        // console.log('event peer:disconnect', evt);
                        // Log disconnected peer
                        var connection = evt.detail;
                        log("Disconnected from ".concat(connection.remotePeer.toString(), " using multiaddr ").concat(connection.remoteAddr.toString()));
                    });
                    if (!argv.relayPeers) return [3 /*break*/, 6];
                    relayPeersFilePath = path.resolve(argv.relayPeers);
                    if (!fs.existsSync(relayPeersFilePath)) {
                        console.log("File at given path ".concat(relayPeersFilePath, " not found, exiting"));
                        process.exit();
                    }
                    console.log("Reading relay peer multiaddr(s) from file ".concat(relayPeersFilePath));
                    relayPeersListObj = fs.readFileSync(relayPeersFilePath, 'utf-8');
                    relayPeersList = JSON.parse(relayPeersListObj);
                    return [4 /*yield*/, _dialRelayPeers(node, relayPeersList)];
                case 5:
                    _a.sent();
                    _a.label = 6;
                case 6: return [2 /*return*/];
            }
        });
    });
}
function _getArgv() {
    return yargs(hideBin(process.argv)).parserConfiguration({
        'parse-numbers': false
    }).options({
        signalServer: {
            type: 'string',
            describe: 'Signalling server URL'
        },
        peerIdFile: {
            type: 'string',
            describe: 'Relay Peer Id file path (json)'
        },
        relayPeers: {
            type: 'string',
            describe: 'Relay peer multiaddr(s) list file path (json)'
        }
    }).argv;
}
function _dialRelayPeers(node, relayPeersList) {
    return __awaiter(this, void 0, void 0, function () {
        var _this = this;
        return __generator(this, function (_a) {
            relayPeersList.forEach(function (relayPeer) { return __awaiter(_this, void 0, void 0, function () {
                var relayMultiaddr;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            relayMultiaddr = multiaddr(relayPeer);
                            console.log("Dialling relay node ".concat(relayMultiaddr.getPeerId(), " using multiaddr ").concat(relayMultiaddr.toString()));
                            return [4 /*yield*/, node.dial(relayMultiaddr)];
                        case 1:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            return [2 /*return*/];
        });
    });
}
main().catch(function (err) {
    console.log(err);
});
//# sourceMappingURL=relay.js.map